# SPDX-FileCopyrightText: 2017-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace, RawDescriptionHelpFormatter
from datetime import date, datetime, timedelta

from gvm.protocols.gmp import Gmp
from terminaltables import AsciiTable


def print_reports(
    gmp: Gmp, from_date: date, to_date: date, reports_choice: str
) -> None:
    host_filter = (
        f"rows=-1 and modified>{from_date.isoformat()} "
        f"and created<{to_date.isoformat()}"
    )

    hosts_xml = gmp.get_hosts(filter_string=host_filter)

    sum_critical = 0
    sum_high = 0
    sum_medium = 0
    sum_low = 0

    if reports_choice == "last":
        table_header = [
            "Hostname",
            "IP",
            "Report",
            "Critical",
            "High",
            "Medium",
            "Low",
        ]
    elif reports_choice == "list":
        table_header = [
            "Hostname",
            "IP",
            "Reports",
            "Critical",
            "High",
            "Medium",
            "Low",
        ]
    else:
        table_header = ["Hostname", "IP", "Critical", "High", "Medium", "Low"]

    table_data = [table_header]

    for host in hosts_xml.xpath("asset"):
        ip = host.xpath("name/text()")[0]

        hostnames = host.xpath(
            'identifiers/identifier/name[text()="hostname"]/../value/text()'
        )

        if len(hostnames) == 0:
            continue

        hostname = hostnames[0]

        results = gmp.get_results(
            details=False,
            filter_string=(
                f"rows=-1 host={ip} and severity>0.0"
                f" and modified>{from_date.isoformat()}"
                f" and modified<{to_date.isoformat()}"
            ),
        )

        unique_vt_results = results.xpath(
            "result["
            "  not (./nvt/@oid = preceding-sibling::result/nvt/@oid)"
            "]"
        )
        if len(unique_vt_results) == 0:
            continue

        low = medium = high = critical = 0
        for result in unique_vt_results:
            threat = result.findtext("threat")
            if threat == "Critical":
                critical += 1
            elif threat == "High":
                high += 1
            elif threat == "Medium":
                medium += 1
            elif threat == "Low":
                low += 1

        sum_low += low
        sum_medium += medium
        sum_high += high
        sum_critical += critical

        if reports_choice == "none":
            table_data.append([hostname, ip, critical, high, medium, low])
        else:
            report_host_identifiers = host.xpath(
                "identifiers/identifier[source/deleted = 0 and"
                '  (source/type = "Report Host"'
                '   or source/type = "Report Host Detail")]'
            )
            report_ids = []
            for identifier in report_host_identifiers:
                mod_date = datetime.fromisoformat(
                    identifier.findtext("modification_time")
                ).date()

                if mod_date >= to_date or mod_date < from_date:
                    continue

                report_ids.append(identifier.find("source").get("id"))
                if reports_choice == "last":
                    break

            if reports_choice == "last":
                table_data.append(
                    [hostname, ip, report_ids[0], critical, high, medium, low]
                )
            else:
                table_data.append(
                    [
                        hostname,
                        ip,
                        ",\n".join(report_ids) + "\n",
                        critical,
                        high,
                        medium,
                        low,
                    ]
                )

    table = AsciiTable(table_data)
    print(f"{table.table}\n")
    print(
        f"Summary of results from {from_date.isoformat()} "
        f"to {to_date.isoformat()}"
    )
    print(f"Critical: {int(sum_critical)}")
    print(f"High: {int(sum_high)}")
    print(f"Medium: {int(sum_medium)}")
    print(f"Low: {int(sum_low)}\n\n")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    description_message = """
This script will display all vulnerabilities from the hosts of the \
reports in a given month and year.
These must be given after the script name as plain numbers.

This version is explicitly made for GOS 24.10.

Example:
    $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/monthly-report2.gmp.py 05 2019
    """

    parser = ArgumentParser(
        prog=("gvm-script [...] " + args.script[0]),
        formatter_class=RawDescriptionHelpFormatter,
        prefix_chars="+",
        description=description_message,
    )
    parser.add_argument("month", type=int, help="month of the monthly report")
    parser.add_argument("year", type=int, help="year of the monthly report")
    parser.add_argument(
        "++reports",
        choices=["none", "last", "list"],
        default="last",
        help=(
            "what to show in the reports column:"
            ' "none": do not show a reports column;'
            ' "last": show the last report in the selected month;'
            ' "list": show a list of reports in the selected month.'
        ),
    )
    script_args, _ = parser.parse_known_args(args.script[1:])

    from_date = date(script_args.year, script_args.month, 1)
    to_date = from_date + timedelta(days=31)
    # To have the first day in month
    to_date = to_date.replace(day=1)

    print_reports(gmp, from_date, to_date, script_args.reports)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table
from datetime import datetime


def list_tasks(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_tasks(details=True, filter_string="rows=-1")
    tasks_xml = response_xml.xpath("task")

    heading = [
        "#",
        "Name",
        "Id",
        "Target",
        "Scanner",
        "Scan Order",
        "Severity",
        "Average Duration",
        "Last Scan Duration (hours)",
    ]

    rows = []
    numberRows = 0

    for task in tasks_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(task.xpath("name/text()"))
        task_id = task.get("id")
        targetname = "".join(task.xpath("target/name/text()"))
        scanner = "".join(task.xpath("scanner/name/text()"))
        severity = "".join(task.xpath("last_report/report/severity/text()"))
        order = "".join(task.xpath("hosts_ordering/text()"))
        average_duration = "".join(task.xpath("average_duration/text()"))
        average_duration_int = (
            0 if not average_duration else int(average_duration)
        )
        average_duration_hours = f"{average_duration_int / 3600:.2f}"
        scan_start_iso = "".join(
            task.xpath("last_report/report/scan_start/text()")
        )
        scan_end_iso = "".join(task.xpath("last_report/report/scan_end/text()"))
        if not scan_start_iso or not scan_end_iso:
            duration_hours = ""
        else:
            scan_start_time = datetime.fromisoformat(
                scan_start_iso.replace("Z", "+00:00")
            )
            scan_end_time = datetime.fromisoformat(
                scan_end_iso.replace("Z", "+00:00")
            )
            duration = scan_end_time - scan_start_time
            duration_hours = f"{duration.total_seconds() / 3600:.2f}"
        rows.append(
            [
                rowNumber,
                name,
                task_id,
                targetname,
                scanner,
                order,
                severity,
                average_duration_hours,
                duration_hours,
            ]
        )

    print(Table(heading=heading, rows=rows))


def main(gmp: Gmp, args: Namespace) -> None:

    print("Listing tasks.\n")

    list_tasks(gmp, args)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script creates a new scan config with nvts from a given CERT-Bund!
        It needs one parameter after the script name.

        1. <cert>   -- Name or ID of the CERT-Bund

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/cfg-gen-for-certs.gmp.py CB-K16/0943
        """
        print(message)
        sys.exit()


WHOLE_ONLY_FAMILIES = [
    "AIX Local Security Checks",
    "AlmaLinux Local Security Checks",
    "Amazon Linux Local Security Checks",
    "Arch Linux Local Security Checks",
    "CentOS Local Security Checks",
    "Debian Local Security Checks",
    "Fedora Local Security Checks",
    "FreeBSD Local Security Checks",
    "Gentoo Local Security Checks",
    "HCE Local Security Checks",
    "HP-UX Local Security Checks",
    "Huawei EulerOS Local Security Checks",
    "Mageia Linux Local Security Checks",
    "Mandrake Local Security Checks",
    "openEuler Local Security Checks",
    "openSUSE Local Security Checks",
    "Oracle Linux Local Security Checks",
    "Red Hat Local Security Checks",
    "Rocky Linux Local Security Checks",
    "Slackware Local Security Checks",
    "Solaris Local Security Checks",
    "SuSE Local Security Checks",
    "Ubuntu Local Security Checks",
    "Windows Local Security Checks",
]


def create_scan_config(gmp, cert_bund_name):
    cert_bund_details = gmp.get_info(
        info_id=cert_bund_name, info_type=gmp.types.InfoType.CERT_BUND_ADV
    )

    list_cves = cert_bund_details.xpath(
        "info/cert_bund_adv/raw_data/Advisory/CVEList/CVE/text()"
    )

    nvt_dict = dict()
    whole_families = set()
    counter = 0

    for cve in list_cves:
        # Get all nvts of this cve
        cve_info = gmp.get_info(info_id=cve, info_type=gmp.types.InfoType.CVE)
        nvts = cve_info.xpath("info/cve/nvts/nvt")

        for nvt in nvts:
            counter += 1
            oid = nvt.xpath("@oid")[0]

            # We need the nvt family to modify scan config
            nvt_data = gmp.get_scan_config_nvt(oid)
            family = nvt_data.xpath("nvt/family/text()")[0]

            # Collect list of whole-only families
            if family in WHOLE_ONLY_FAMILIES:
                whole_families.add(family)
            # Create key value map
            elif family in nvt_dict and oid not in nvt_dict[family]:
                nvt_dict[family].append(oid)
            else:
                nvt_dict[family] = [oid]

    # Create new config
    copy_id = "085569ce-73ed-11df-83c3-002264764cea"
    config_name = f"scanconfig_for_{cert_bund_name}"
    config_id = ""

    try:
        res = gmp.create_scan_config(copy_id, config_name)
        config_id = res.xpath("@id")[0]

        # Modify the config with the nvts oid
        for family, nvt_oid in nvt_dict.items():
            try:
                gmp.modify_scan_config_set_nvt_selection(
                    config_id=config_id, nvt_oids=nvt_oid, family=family
                )
            except GvmError as gvmerr:
                if (
                    "Attempt to modify NVT in whole-only family"
                    in gvmerr.message
                ):
                    print(
                        f'Adding whole family "{family}" to scan config, '
                        f"which is not present in WHOLE_ONLY_FAMILIES"
                    )
                    whole_families.add(family)
                else:
                    print(f"Could not modify scan config, {gvmerr=}")

        if whole_families:
            print(f"Adding whole families to scan config: {whole_families}")

            gmp.modify_scan_config_set_family_selection(
                config_id=config_id,
                families=[(f, True, True) for f in whole_families],
            )

        # This nvts must be present to work
        family = "Port scanners"
        nvts = ["1.3.6.1.4.1.25623.1.0.14259", "1.3.6.1.4.1.25623.1.0.100315"]
        gmp.modify_scan_config_set_nvt_selection(
            config_id=config_id, nvt_oids=nvts, family=family
        )

        print("Finished")

    except GvmError as e:
        print("Config exist ", e)


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    cert_bund_name = args.script[1]

    print(f"Creating scan config for {cert_bund_name}")

    create_scan_config(gmp, cert_bund_name)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2025 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Loosely based on other greenbone scripts
#
# Run with: gvm-script --gmp-username admin-user --gmp-password password socket export-hosts-csv.gmp.py <csv file> days
# example: gvm-script --gmp-username admin --gmp-password top$ecret socket export-hosts-csv.gmp.py hosts.csv 2

import csv
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import date, datetime, time, timedelta

# GVM Specific
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script generates a csv file with certificates "
    "from Greenbone Vulnerability Manager.\n"
    "Optional: Specify csv filename and days in the past to get data from as positional arguments\n"
    "or live with the defaults\n"
    "csv file will contain:\n"
    "Subject, Issuer, Serial, SHA256 Fingerprint, MD5 Fingerprint, last_seen, Valid From, Valid To"
)


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "csv_filename",
        nargs="?",
        default="gvm_certificates.csv",
        type=str,
        help=(
            "Optional: CSV File with certificate information - Default: gvm_certificates.csv"
        ),
    )

    parser.add_argument(
        "delta_days",
        nargs="?",
        default="1",
        type=int,
        help=(
            "Optional: Number of days in the past to pull information - Default: 1 day"
        ),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def list_tls_certificates(
    gmp: Gmp, from_date: date, to_date: date, csvfilename: str
) -> None:
    tls_cert_filter = (
        f"rows=-1 and modified>{from_date.isoformat()} "
        f"and modified<{to_date.isoformat()}"
    )

    cert_info = []

    tls_cert_xml = gmp.get_tls_certificates(filter_string=tls_cert_filter)
    # pretty_print(tls_cert_xml)

    for tls_cert in tls_cert_xml.xpath("tls_certificate"):

        cert_seen = tls_cert.xpath("last_seen/text()")[0]

        cert_from = tls_cert.xpath("activation_time/text()")[0]

        cert_to = tls_cert.xpath("expiration_time/text()")[0]

        cert_subject = tls_cert.xpath("subject_dn/text()")[0]

        cert_issuer = tls_cert.xpath("issuer_dn/text()")[0]

        cert_serial = tls_cert.xpath("serial/text()")[0]

        cert_sha256 = tls_cert.xpath("sha256_fingerprint/text()")[0]

        cert_md5 = tls_cert.xpath("md5_fingerprint/text()")[0]

        cert_info.append(
            [
                cert_subject,
                cert_issuer,
                cert_serial,
                cert_sha256,
                cert_md5,
                cert_seen,
                cert_from,
                cert_to,
            ]
        )

    # Write the list host_info to csv file
    writecsv(csvfilename, cert_info)
    print(
        f"CSV file: {csvfilename}\n"
        f"From:     {from_date}\n"
        f"To:       {to_date}\n"
    )


def writecsv(csv_filename: str, hostinfo: list) -> None:
    field_names = [
        "Subject",
        "Issuer",
        "Serial",
        "SHA256 Fingerprint",
        "MD5 Fingerprint",
        "Last seen",
        "Valid From",
        "Valid To",
    ]
    try:
        with open(csv_filename, "w") as csvfile:
            writer = csv.writer(csvfile, delimiter=",", quoting=csv.QUOTE_ALL)
            writer.writerow(field_names)
            writer.writerows(hostinfo)
            csvfile.close
    except IOError as e:
        error_and_exit(f"Failed to write csv file: {str(e)} (exit)")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    args = args.script[1:]
    parsed_args = parse_args(args=args)
    csv_filename = parsed_args.csv_filename
    delta_days = parsed_args.delta_days

    # simply getting yesterday from midnight to now
    from_date = datetime.combine(datetime.today(), time.min) - timedelta(
        days=delta_days
    )
    to_date = datetime.now()
    # get the certs
    list_tls_certificates(gmp, from_date, to_date, csv_filename)


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see<http://www.gnu.org/licenses/>


import unittest
from argparse import Namespace
from pathlib import Path
from unittest.mock import patch

from gvmtools.helper import Table

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class ListTasksTestCase(unittest.TestCase):
    def setUp(self):
        self.list_tasks = load_script(
            (CWD.parent.parent / "scripts"), "list-tasks"
        )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    @patch("builtins.print")
    def test_duration_with_timezone_offset(
        self, mock_print, mock_gmp: GmpMockFactory
    ):
        args = Namespace(script=["foo"])

        mock_gmp.mock_response(
            "get_tasks",
            '<get_tasks_response status="200" status_text="OK">'
            '<task id="7f32c1f4-c5c8-4035-b4fb-5b1e220bc052">'
            "<name>Test</name>"
            "<average_duration>4115</average_duration>"
            "<last_report>"
            '<report id="cbf8c159-d3f8-4adf-b17d-fd53abf20e52">'
            "<scan_start>2024-03-12T16:35:01-07:00</scan_start>"
            "<scan_end>2024-03-12T17:43:36-07:00</scan_end>"
            "</report>"
            "</last_report>"
            "</task>"
            "</get_tasks_response>",
        )
        self.list_tasks.list_tasks(mock_gmp.gmp_protocol, args)

        table = mock_print.call_args[0][0]
        self.assertIsInstance(table, Table)
        self.assertEqual(len(table.rows), 1)
        self.assertEqual(table.rows[0][1], "Test")
        self.assertEqual(table.rows[0][7], "1.14")
        self.assertEqual(table.rows[0][8], "1.14")

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    @patch("builtins.print")
    def test_duration_in_UTC(self, mock_print, mock_gmp: GmpMockFactory):
        args = Namespace(script=["foo"])

        mock_gmp.mock_response(
            "get_tasks",
            '<get_tasks_response status="200" status_text="OK">'
            '<task id="7f32c1f4-c5c8-4035-b4fb-5b1e220bc052">'
            "<name>Test</name>"
            "<average_duration>14444</average_duration>"
            "<last_report>"
            '<report id="cbf8c159-d3f8-4adf-b17d-fd53abf20e52">'
            "<scan_start>2024-09-27T14:30:01Z</scan_start>"
            "<scan_end>2024-09-27T18:30:45Z</scan_end>"
            "</report>"
            "</last_report>"
            "</task>"
            "</get_tasks_response>",
        )
        self.list_tasks.list_tasks(mock_gmp.gmp_protocol, args)

        table = mock_print.call_args[0][0]
        self.assertIsInstance(table, Table)
        self.assertEqual(len(table.rows), 1)
        self.assertEqual(table.rows[0][1], "Test")
        self.assertEqual(table.rows[0][7], "4.01")
        self.assertEqual(table.rows[0][8], "4.01")

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    @patch("builtins.print")
    def test_missing_last_report(self, mock_print, mock_gmp: GmpMockFactory):
        args = Namespace(script=["foo"])

        mock_gmp.mock_response(
            "get_tasks",
            '<get_tasks_response status="200" status_text="OK">'
            '<task id="7f32c1f4-c5c8-4035-b4fb-5b1e220bc052">'
            "<name>Test</name>"
            "<average_duration>0</average_duration>"
            "</task>"
            "</get_tasks_response>",
        )
        self.list_tasks.list_tasks(mock_gmp.gmp_protocol, args)

        table = mock_print.call_args[0][0]
        self.assertIsInstance(table, Table)
        self.assertEqual(len(table.rows), 1)
        self.assertEqual(table.rows[0][1], "Test")
        self.assertEqual(table.rows[0][7], "0.00")
        self.assertEqual(table.rows[0][8], "")



# SPDX-FileCopyrightText: 2025 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Loosely based on other greenbone scripts
#
# Run with: gvm-script --gmp-username admin-user --gmp-password password socket list-hosts.gmp.py <days>
# example: gvm-script --gmp-username admin --gmp-password top$ecret socket list-hosts.gmp.py 2


import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import date, datetime, time, timedelta

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table

HELP_TEXT = (
    "This script generates a table of hosts (assets) "
    "from Greenbone Vulnerability Manager.\n\n"
    "table will contain:\n"
    "Hostname, IP Address, MAC Address, Operating System, last seen, and severity\n"
)


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 1:
        message = """
        This script requests information about all hosts <days> days prior to today (included) and 
        displays it as a table. It requires one parameter after the script name:
        1. days -- number of days prior to today to pull hosts information from

        Examples:
            $ gvm-script --gmp-username username --gmp-password password socket list-hosts.gmp.py <days>
            $ gvm-script --gmp-username admin --gmp-password 0f6fa69b-32bb-453a-9aa4-b8c9e56b3d00 socket list-hosts.gmp.py 4
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "delta_days",
        type=int,
        help=("Number of days in the past to pull hosts information"),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def list_hosts(gmp: Gmp, from_date: date, to_date: date) -> None:
    host_filter = (
        f"rows=-1 "
        f"and modified>{from_date.isoformat()} "
        f"and modified<{to_date.isoformat()}"
    )

    # Get the XML of hosts
    hosts_xml = gmp.get_hosts(filter_string=host_filter)
    heading = [
        "#",
        "IP Address",
        "Hostname",
        "MAC Address",
        "Operating System",
        "Last Seen",
        "CVSS",
    ]
    rows = []
    numberRows = 0

    print("Listing hosts.\n" f"From: {from_date}\n" f"To:   {to_date}\n")

    for host in hosts_xml.xpath("asset"):
        # ip will always be there
        ip = host.xpath("name/text()")[0]
        host_seendates = host.xpath("modification_time/text()")
        host_lastseen = host_seendates[0]

        # hostnames and other attributes may not be there
        hostnames = host.xpath(
            'identifiers/identifier/name[text()="hostname"]/../value/text()'
        )
        if len(hostnames) == 0:
            hostname = ""
            pass
        else:
            hostname = hostnames[0]

        host_macs = host.xpath(
            'identifiers/identifier/name[text()="MAC"]/../value/text()'
        )
        if len(host_macs) == 0:
            host_mac = ""
            pass
        else:
            host_mac = host_macs[0]

        host_severity = host.xpath("host/severity/value/text()")
        if len(host_severity) == 0:
            host_severity = "0.0"
            pass
        else:
            host_severity = host_severity[0]

        host_os = host.xpath(
            'host/detail/name[text()="best_os_txt"]/../value/text()'
        )
        if len(host_os) == 0:
            host_os = ""
            pass
        else:
            host_os = host_os[0]
        # Count number of hosts
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)
        rows.append(
            [
                rowNumber,
                ip,
                hostname,
                host_mac,
                host_os,
                host_lastseen,
                host_severity,
            ]
        )

    print(Table(heading=heading, rows=rows))


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    check_args(args)
    if args.script:
        args = args.script[1:]
    parsed_args = parse_args(args=args)
    delta_days = parsed_args.delta_days
    # simply getting yesterday from midnight to today (now)
    from_date = datetime.combine(datetime.today(), time.min) - timedelta(
        days=delta_days
    )
    to_date = datetime.now()
    # print(from_date, to_date)

    list_hosts(gmp, from_date, to_date)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2025 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Loosely based on other greenbone scripts
#
# Run with: gvm-script --gmp-username admin-user --gmp-password password socket export-hosts-csv.gmp.py <csv file> days
# example: gvm-script --gmp-username admin --gmp-password top$ecret socket export-hosts-csv.gmp.py hosts.csv 2


import csv
import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import date, datetime, time, timedelta

from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script generates a csv file with hosts (assets) "
    "from Greenbone Vulnerability Manager.\n\n"
    "csv file will contain:\n"
    "IP Address, Hostname, MAC Address, Operating System, last seen, and severity\n"
)


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script requests all hosts <days> prior to today and exports it as a csv file.
        It requires two parameter after the script name:
        1. filename -- name of the csv file of the report
        2. days     -- number of days before and until today to pull hosts information from
        
        Examples:
            $ gvm-script --gmp-username username --gmp-password password socket export-hosts-csv.gmp.py <csv_file> <days>
            $ gvm-script --gmp-username admin --gmp-password 0f6fa69b-32bb-453a-9aa4-b8c9e56b3d00 socket export-hosts-csv.gmp.py hosts.csv 4
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "csv_filename",
        type=str,
        help=("CSV File containing credentials"),
    )

    parser.add_argument(
        "delta_days",
        type=int,
        help=("Number of days in the past to pull hosts information"),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def list_hosts(
    gmp: Gmp, from_date: date, to_date: date, csvfilename: str
) -> None:
    host_filter = (
        f"rows=-1 and modified>{from_date.isoformat()} "
        f"and modified<{to_date.isoformat()}"
    )

    # Get the XML of hosts
    hosts_xml = gmp.get_hosts(filter_string=host_filter)
    host_info = []

    for host in hosts_xml.xpath("asset"):
        # ip will always be there
        ip = host.xpath("name/text()")[0]
        host_seendates = host.xpath("modification_time/text()")
        host_lastseen = host_seendates[0]

        # hostnames and other attributes may not be there
        hostnames = host.xpath(
            'identifiers/identifier/name[text()="hostname"]/../value/text()'
        )
        if len(hostnames) == 0:
            hostname = ""
            pass
        else:
            hostname = hostnames[0]

        host_macs = host.xpath(
            'identifiers/identifier/name[text()="MAC"]/../value/text()'
        )
        if len(host_macs) == 0:
            host_mac = ""
            pass
        else:
            host_mac = host_macs[0]

        host_severity = host.xpath("host/severity/value/text()")
        if len(host_severity) == 0:
            host_severity = 0
            pass
        else:
            host_severity = host_severity[0]

        host_os = host.xpath(
            'host/detail/name[text()="best_os_txt"]/../value/text()'
        )
        if len(host_os) == 0:
            host_os = ""
            pass
        else:
            host_os = host_os[0]

        host_info.append(
            [hostname, ip, host_mac, host_os, host_lastseen, host_severity]
        )
    # Write the list host_info to csv file
    writecsv(csvfilename, host_info)
    print(
        f"CSV file: {csvfilename}\n"
        f"From:     {from_date}\n"
        f"To:       {to_date}\n"
    )


def writecsv(csv_filename, hostinfo: list) -> None:
    field_names = [
        "IP Address",
        "Hostname",
        "MAC Address",
        "Operating System",
        "Last Seen",
        "CVSS",
    ]
    try:
        with open(csv_filename, "w") as csvfile:
            writer = csv.writer(csvfile, delimiter=",", quoting=csv.QUOTE_ALL)
            writer.writerow(field_names)
            writer.writerows(hostinfo)
            csvfile.close
    except IOError as e:
        error_and_exit(f"Failed to write csv file: {str(e)} (exit)")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    # argv[0] contains the csv file name
    check_args(args)
    if args.script:
        args = args.script[1:]
    parsed_args = parse_args(args=args)

    delta_days = parsed_args.delta_days
    # simply getting yesterday from midnight to now
    from_date = datetime.combine(datetime.today(), time.min) - timedelta(
        days=delta_days
    )
    to_date = datetime.now()
    # get the hosts
    list_hosts(gmp, from_date, to_date, parsed_args.csv_filename)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2025 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Loosely based on other greenbone scripts
#
# Run with: gvm-script --gmp-username admin-user --gmp-password password socket export-hosts-csv.gmp.py <csv file> days
# example: gvm-script --gmp-username admin --gmp-password top$ecret socket export-hosts-csv.gmp.py hosts.csv 2


import csv
import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import date, datetime, time, timedelta

from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script generates a csv file with Operating System "
    "from Greenbone Vulnerability Manager.\n\n"
    "csv file will contain:\n"
    "IP Address, Hostname, MAC Address, Operating System, Last Seen, CVSS\n"
)


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script requests all hosts <days> prior to today and exports it as a csv file.
        It requires two parameter after the script name:
        1. filename -- name of the csv file of the report
        2. days     -- number of days before and until today to pull hosts information from
        
        Examples:
            $ gvm-script --gmp-username username --gmp-password password socket export-hosts-csv.gmp.py <csv_file> <days>
            $ gvm-script --gmp-username admin --gmp-password 0f6fa69b-32bb-453a-9aa4-b8c9e56b3d00 socket export-hosts-csv.gmp.py operatingsystems.csv 4
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "csv_filename",
        type=str,
        help=("CSV File containing credentials"),
    )

    parser.add_argument(
        "delta_days",
        type=int,
        help=("Number of days in the past to pull hosts information"),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def list_operating_systems(
    gmp: Gmp, from_date: date, to_date: date, csvfilename: str
) -> None:
    operating_system_filter = (
        f"rows=-1 "
        f"and modified>{from_date.isoformat()} "
        f"and modified<{to_date.isoformat()}"
    )

    os_info = []

    operating_systems_xml = gmp.get_operating_systems(
        filter_string=operating_system_filter
    )

    for operating_system in operating_systems_xml.xpath("asset"):
        # title will always be there
        os_title = operating_system.xpath("name/text()")[0]

        os_latest_severity = operating_system.xpath(
            "os/latest_severity/value/text()"
        )[0]

        os_host_count = operating_system.xpath("os/installs/text()")[0]

        os_info.append(
            [
                os_title,
                os_latest_severity,
                os_host_count,
            ]
        )

    # Write the list host_info to csv file
    writecsv(csvfilename, os_info)
    print(
        f"CSV file: {csvfilename}\n"
        f"From:     {from_date}\n"
        f"To:       {to_date}\n"
    )


def writecsv(csv_filename: str, hostinfo: list) -> None:
    field_names = [
        "IP Address",
        "Hostname",
        "MAC Address",
        "Operating System",
        "Last Seen",
        "CVSS",
    ]
    try:
        with open(csv_filename, "w") as csvfile:
            writer = csv.writer(csvfile, delimiter=",", quoting=csv.QUOTE_ALL)
            writer.writerow(field_names)
            writer.writerows(hostinfo)
            csvfile.close
    except IOError as e:
        error_and_exit(f"Failed to write csv file: {str(e)} (exit)")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    # argv[0] contains the csv file name
    check_args(args)
    if args.script:
        args = args.script[1:]
    parsed_args = parse_args(args=args)

    delta_days = parsed_args.delta_days
    # simply getting yesterday from midnight to now
    from_date = datetime.combine(datetime.today(), time.min) - timedelta(
        days=delta_days
    )
    to_date = datetime.now()
    # get the hosts
    list_operating_systems(gmp, from_date, to_date, parsed_args.csv_filename)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2025 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Loosely based on other greenbone scripts
#
# Run with: gvm-script --gmp-username admin-user --gmp-password password socket verify-scanners.gmp.py

from argparse import Namespace

from gvm.errors import GvmServerError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    heading = ["#", "Name", "Id", "Host", "Version"]

    rows = []
    numberRows = 0

    print("Verifying scanners.\n")
    response_xml = gmp.get_scanners(filter_string="rows=-1")
    scanners_xml = response_xml.xpath("scanner")

    for scanner in scanners_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)
        name = "".join(scanner.xpath("name/text()"))
        scanner_id = scanner.get("id")
        host = "".join(scanner.xpath("host/text()"))
        if host == "":
            host = "local scanner"
        try:
            status_xml = gmp.verify_scanner(str(scanner_id))
            # pretty_print(status_xml)
            for scanner_status in status_xml:
                scanner_version = "".join(scanner_status.xpath("text()"))
        except GvmServerError:
            scanner_version = "*No Response*"
            pass

        rows.append([rowNumber, name, scanner_id, host, scanner_version])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from pathlib import Path

from gvm.protocols.gmp import Gmp
from lxml import etree


def check_args(args):
    len_args = len(args.script) - 1
    if len_args < 1:
        message = """
        This script requests the given report and exports it as a xml
        file locally. It requires one parameters after the script name.

        1. <report_id>     -- ID of the report

        Optional a file name to save the xml in.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/export-xml-report.gmp.py <report_id> <xml_file>
        """
        print(message)
        sys.exit()


def main(gmp: Gmp, args: Namespace) -> None:
    # check if report id and xml filename are provided to the script
    # argv[0] contains the script name
    check_args(args)

    report_id = args.argv[1]
    if len(args.argv) == 3:
        xml_filename = args.argv[2] + ".xml"
    else:
        xml_filename = args.argv[1] + ".xml"

    xml_report_format_id = "5057e5cc-b825-11e4-9d0e-28d24461215b"

    response = gmp.get_report(
        report_id=report_id,
        report_format_id=xml_report_format_id,
        ignore_pagination=True,
        details=True,
    )

    report_element = response.find("report")
    # get the full content of the report element as bytestring
    content = etree.tostring(report_element.find("report"))

    if not content:
        print(
            "Requested report is empty. Either the report does not contain any"
            " results or the necessary tools for creating the report are "
            "not installed.",
            file=sys.stderr,
        )
        sys.exit(1)

    # decode content
    dcontent = content.decode("utf-8")

    # write to file and support ~ in filename path
    xml_path = Path(xml_filename).expanduser()

    xml_path.write_text(dcontent)

    print("Done. xml created: " + str(xml_path))


if __name__ == "__gmp__":
    main(gmp, args)
